package com.github.clevernucleus.playerex.api.client;

import java.text.DecimalFormat;
import java.util.List;
import net.minecraft.class_124;
import net.minecraft.class_1320;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_2960;
import net.minecraft.class_5250;
import com.github.clevernucleus.dataattributes.api.API;
import com.github.clevernucleus.dataattributes.api.attribute.AttributeBehaviour;
import com.github.clevernucleus.dataattributes.api.attribute.IAttribute;
import com.github.clevernucleus.dataattributes.api.attribute.IAttributeFunction;
import com.github.clevernucleus.playerex.api.ExAPI;

/**
 * A helper class providing useful methods to use with PlayerEx.
 * 
 * @author CleverNucleus
 *
 */
public final class ClientUtil {
	
	/** Format for two decimal places. */
	public static final DecimalFormat FORMATTING_2 = new DecimalFormat("###.##");
	
	/** Format for three decimal places. */
	public static final DecimalFormat FORMATTING_3 = new DecimalFormat("###.###");
	
	/**
	 * Takes the input attribute and looks for the {@link ExAPI#PERCENTAGE_PROPERTY} property; if present, multiplies the input 
	 * value by the property's multiplier. If not, then looks for the {@link ExAPI#MULTIPLIER_PROPERTY} property; if present, 
	 * multiplies the input value by the property's multiplier. Else returns the input value.
	 * @param attributeIn
	 * @param valueIn
	 * @return
	 */
	public static double displayValue(final IAttribute attributeIn, double valueIn) {
		if(attributeIn.hasProperty(ExAPI.PERCENTAGE_PROPERTY)) return valueIn * ExAPI.parse(attributeIn.getProperty(ExAPI.PERCENTAGE_PROPERTY));
		if(attributeIn.hasProperty(ExAPI.MULTIPLIER_PROPERTY)) return valueIn * ExAPI.parse(attributeIn.getProperty(ExAPI.MULTIPLIER_PROPERTY));
		return valueIn;
	}
	
	/**
	 * If the input value is positive, adds the "+" prefix. If the input attribute has the {@link ExAPI#PERCENTAGE_PROPERTY}
	 * property, appends the "%" suffix.
	 * @param attributeIn
	 * @param valueIn
	 * @return
	 */
	public static String formatValue(final IAttribute attributeIn, double valueIn) {
		String value = FORMATTING_3.format(valueIn);
		
		if(valueIn > 0) {
			value = "+" + value;
		}
		
		if(attributeIn.hasProperty(ExAPI.PERCENTAGE_PROPERTY)) {
			value = value + "%";
		}
		
		return value;
	}
	
	/**
	 * Looks at the input attribute's functions, and if there are any, formats them and adds them to the input list as tooltips.
	 * @param tooltip
	 * @param attributeIn
	 */
	public static void appendFunctionsToTooltip(List<class_2561> tooltip, final class_1320 attributeIn) {
		if(attributeIn == null) return;
		
		IAttribute attribute = (IAttribute)attributeIn;
		
		for(IAttributeFunction function : attribute.functions()) {
			class_2960 identifier = function.attribute();
			class_1320 subAttributeIn = API.getAttribute(identifier).get();
			
			if(subAttributeIn == null) continue;
			
			IAttribute subAttribute = (IAttribute)subAttributeIn;
			double displayValue = displayValue(subAttribute, function.multiplier());
			String formattedValue = formatValue(subAttribute, displayValue);
			class_5250 mutableText = new class_2585(formattedValue + " ");
			mutableText.method_10852(new class_2588(subAttributeIn.method_26830()));
			
			if(function.behaviour().equals(AttributeBehaviour.DIMINISHING)) {
				mutableText.method_10852(new class_2588("gui.playerex.text.diminishing"));
			}
			
			tooltip.add(mutableText.method_27692(class_124.field_1080));
		}
	}
}
