package com.github.clevernucleus.playerex.api.event;

import org.jetbrains.annotations.Nullable;

import com.google.common.collect.Multimap;

import net.fabricmc.fabric.api.event.Event;
import net.fabricmc.fabric.api.event.EventFactory;
import net.minecraft.class_1304;
import net.minecraft.class_1320;
import net.minecraft.class_1322;
import net.minecraft.class_1792;
import net.minecraft.class_1799;

/**
 * Provides a hook into the ItemStack constructor and ItemStack#getAttributeModifiers.
 * 
 * @author CleverNucleus
 *
 */
public final class ItemStackEvents {
	
	/**
	 * Fired at the end of the itemstack constructor. The item may be null.
	 */
	public static final Event<ItemStackEvents.Constructor> ITEMSTACK_CONSTRUCTED = EventFactory.createArrayBacked(ItemStackEvents.Constructor.class, listeners -> (item, itemStack, count) -> {
		for(Constructor listener : listeners) {
			listener.onItemStackConstructor(item, itemStack, count);
		}
	});
	
	/**
	 * Fired on {@link class_1799#method_7926(class_1304)}. Allows for editing the modifiers multimap with ItemStack (i.e. NBT) arguments.
	 */
	public static final Event<ItemStackEvents.GetAttributeModifiers> ITEMSTACK_MODIFIERS = EventFactory.createArrayBacked(ItemStackEvents.GetAttributeModifiers.class, listeners -> (itemStack, modifiers, slot) -> {
		for(GetAttributeModifiers listener : listeners) {
			listener.getAttributeModifiers(itemStack, modifiers, slot);
		}
	});
	
	@FunctionalInterface
	public interface Constructor {
		
		/**
		 * 
		 * @param item
		 * @param itemStack
		 * @param count
		 */
		void onItemStackConstructor(@Nullable final class_1792 item, final class_1799 itemStack, final int count);
	}
	
	@FunctionalInterface
	public interface GetAttributeModifiers {
		
		/**
		 * 
		 * @param itemStack
		 * @param modifiers
		 * @param slot
		 */
		void getAttributeModifiers(final class_1799 itemStack, final Multimap<class_1320, class_1322> modifiers, final class_1304 slot);
	}
}
