package com.github.clevernucleus.playerex.api.event;

import net.fabricmc.fabric.api.event.Event;
import net.fabricmc.fabric.api.event.EventFactory;
import net.minecraft.class_1269;
import net.minecraft.class_3222;

/**
 * Provides a hook into the method {@link PlayerEntity#addExperience(int)}.
 * 
 * @author CleverNucleus
 *
 */
public final class PlayerLevelUpEvent {
	
	/**
	 * Fired ONCE on the server every time the player first gains enough xp to reach the next level. 
	 * For example:
	 * <br></br>
	 * If the player needs two experience levels to level up, then the instant the player first gains two experience levels, this event fires.
	 * If however, the player then continues to gain experience levels past the required number, this event does not fire again until the 
	 * player has skilled an attribute and the experience levels once again drop below the required number.
	 */
	public static final Event<PlayerLevelUpEvent.LevelUp> EVENT = EventFactory.createArrayBacked(PlayerLevelUpEvent.LevelUp.class, listeners -> player -> {
		for(LevelUp listener : listeners) {
			class_1269 result = listener.onLevelUp(player);
			
			if(result != class_1269.field_5811) {
				return result;
			}
		}
		
		return class_1269.field_5811;
	});
	
	@FunctionalInterface
	public interface LevelUp {
		
		/**
		 * 
		 * @param player
		 * @return
		 */
		class_1269 onLevelUp(final class_3222 player);
	}
}
